# Generated by Django 5.0.6 on 2024-06-04 12:05
from django.db import migrations

import core.fields.image


class Migration(migrations.Migration):
    dependencies = [
        ("blog", "0014_alter_blogposttranslation_search_document_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="blogcategory",
            name="image",
            field=core.fields.image.ImageAndSvgField(
                blank=True,
                null=True,
                upload_to="uploads/blog/",
                verbose_name="Image",
            ),
        ),
    ]
