# Generated by Django 5.0.4 on 2024-04-11 17:27
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0005_alter_orderitem_options"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="orderitem",
            options={
                "ordering": ["sort_order"],
                "verbose_name": "Order Item",
                "verbose_name_plural": "Order Items",
            },
        ),
    ]
