# Generated by Django 5.1.4 on 2024-12-31 00:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0011_alter_useraccount_country_alter_useraccount_region_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='useraccount',
            name='address',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Address'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='bio',
            field=models.TextField(blank=True, default='', verbose_name='Bio'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='city',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='City'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='facebook',
            field=models.URLField(blank=True, default='', verbose_name='Facebook Profile'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='first_name',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='First Name'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='github',
            field=models.URLField(blank=True, default='', verbose_name='Github Profile'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='instagram',
            field=models.URLField(blank=True, default='', verbose_name='Instagram Profile'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='last_name',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Last Name'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='linkedin',
            field=models.URLField(blank=True, default='', verbose_name='LinkedIn Profile'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='place',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Place'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='twitter',
            field=models.URLField(blank=True, default='', verbose_name='Twitter Profile'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='website',
            field=models.URLField(blank=True, default='', verbose_name='Website'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='youtube',
            field=models.URLField(blank=True, default='', verbose_name='Youtube Profile'),
        ),
        migrations.AlterField(
            model_name='useraccount',
            name='zipcode',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Zip Code'),
        ),
        migrations.AlterField(
            model_name='useraddress',
            name='notes',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Notes'),
        ),
    ]
