# Generated by Django 5.0.3 on 2024-03-29 20:53
import django.db.models.deletion
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        (
            "blog",
            "0003_blogcomment_level_blogcomment_lft_blogcomment_parent_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="blogpost",
            name="author",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="posts",
                to="blog.blogauthor",
            ),
        ),
        migrations.AlterField(
            model_name="blogpost",
            name="category",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="posts",
                to="blog.blogcategory",
            ),
        ),
        migrations.AlterField(
            model_name="blogpost",
            name="tags",
            field=models.ManyToManyField(
                blank=True, related_name="tags", to="blog.blogtag"
            ),
        ),
    ]
