# Generated by Django 5.0.3 on 2024-03-31 12:31
import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("country", "0001_initial"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="country",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["created_at"], name="country_created_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="country",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["updated_at"], name="country_updated_at_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="country",
            index=django.contrib.postgres.indexes.BTreeIndex(
                fields=["sort_order"], name="country_sort_order_idx"
            ),
        ),
    ]
