# Generated by Django 5.0.4 on 2024-04-11 11:43
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("blog", "0007_remove_blogcomment_unique_blog_comment"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="blogcategory",
            options={
                "ordering": ["-sort_order"],
                "verbose_name": "Blog Category",
                "verbose_name_plural": "Blog Categories",
            },
        ),
        migrations.AlterModelOptions(
            name="blogtag",
            options={
                "ordering": ["-sort_order"],
                "verbose_name": "Blog Tag",
                "verbose_name_plural": "Blog Tags",
            },
        ),
    ]
