# Generated by Django 5.2.2 on 2025-06-08 22:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tag', '0004_remove_taggeditem_tag_taggedi_content_cd6f88_idx_and_more'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='tag',
            managers=[
            ],
        ),
    ]
