# Generated by Django 5.1 on 2024-08-24 22:51
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("pay_way", "0005_alter_payway_cost_alter_payway_free_for_order_amount"),
    ]

    operations = [
        migrations.AlterField(
            model_name="payway",
            name="sort_order",
            field=models.IntegerField(
                editable=False, null=True, verbose_name="Sort Order"
            ),
        ),
    ]
