# Generated by Django 5.2.1 on 2025-05-14 20:33

import django.contrib.postgres.indexes
from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('blog', '0024_alter_blogauthor_website_and_more'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RenameIndex(
            model_name='blogauthor',
            new_name='blogauthor_created_at_ix',
            old_name='blogauthor_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='blogauthor',
            new_name='blogauthor_updated_at_ix',
            old_name='blogauthor_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='blogcategory',
            new_name='blogcategory_created_at_ix',
            old_name='blogcategory_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='blogcategory',
            new_name='blogcategory_updated_at_ix',
            old_name='blogcategory_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='blogcategory',
            new_name='blogcategory_sort_order_ix',
            old_name='blogcategory_sort_order_idx',
        ),
        migrations.RenameIndex(
            model_name='blogcomment',
            new_name='blogcomment_created_at_ix',
            old_name='blogcomment_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='blogcomment',
            new_name='blogcomment_updated_at_ix',
            old_name='blogcomment_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='blogpost',
            new_name='blogpost_created_at_ix',
            old_name='blogpost_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='blogpost',
            new_name='blogpost_updated_at_ix',
            old_name='blogpost_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='blogpost',
            new_name='blogpost_published_at_ix',
            old_name='blogpost_published_at_idx',
        ),
        migrations.RenameIndex(
            model_name='blogpost',
            new_name='blogpost_is_published_ix',
            old_name='blogpost_is_published_idx',
        ),
        migrations.RenameIndex(
            model_name='blogpost',
            new_name='blog_post_view_count_ix',
            old_name='blog_blogpo_view_co_226064_btree',
        ),
        migrations.RenameIndex(
            model_name='blogpost',
            new_name='blog_post_status_ix',
            old_name='blog_blogpo_status_e66397_btree',
        ),
        migrations.RenameIndex(
            model_name='blogpost',
            new_name='blog_post_featured_ix',
            old_name='blog_blogpo_feature_b84efc_btree',
        ),
        migrations.RenameIndex(
            model_name='blogpost',
            new_name='blog_post_category_ix',
            old_name='blog_blogpo_categor_4d71d3_btree',
        ),
        migrations.RenameIndex(
            model_name='blogpost',
            new_name='blog_post_author_ix',
            old_name='blog_blogpo_author__753b9d_btree',
        ),
        migrations.RenameIndex(
            model_name='blogtag',
            new_name='blogtag_created_at_ix',
            old_name='blogtag_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='blogtag',
            new_name='blogtag_updated_at_ix',
            old_name='blogtag_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='blogtag',
            new_name='blogtag_sort_order_ix',
            old_name='blogtag_sort_order_idx',
        ),
        migrations.AddIndex(
            model_name='blogauthor',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['user'], name='blog_author_user_ix'),
        ),
        migrations.AddIndex(
            model_name='blogcategory',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['slug'], name='blog_category_slug_ix'),
        ),
        migrations.AddIndex(
            model_name='blogcategory',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['parent'], name='blog_category_parent_ix'),
        ),
        migrations.AddIndex(
            model_name='blogcomment',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['is_approved'], name='blog_blogco_is_appr_b3338c_btree'),
        ),
        migrations.AddIndex(
            model_name='blogcomment',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['post'], name='blog_blogco_post_id_8fff6c_btree'),
        ),
        migrations.AddIndex(
            model_name='blogcomment',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['user'], name='blog_blogco_user_id_78a803_btree'),
        ),
        migrations.AddIndex(
            model_name='blogcomment',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['parent'], name='blog_blogco_parent__6c614f_btree'),
        ),
        migrations.AddIndex(
            model_name='blogpost',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['slug'], name='blog_post_slug_ix'),
        ),
        migrations.AddIndex(
            model_name='blogpost',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['status', 'featured'], name='blog_post_status_featured_ix'),
        ),
    ]
