# Generated by Django 5.1.3 on 2024-11-27 19:34
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cart", "0006_alter_cartitem_options"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="cart",
            name="cart_created_at_idx",
        ),
        migrations.RemoveIndex(
            model_name="cart",
            name="cart_updated_at_idx",
        ),
        migrations.RemoveIndex(
            model_name="cart",
            name="cart_cart_user_id_0a6acd_btree",
        ),
        migrations.RemoveIndex(
            model_name="cartitem",
            name="cartitem_created_at_idx",
        ),
        migrations.RemoveIndex(
            model_name="cartitem",
            name="cartitem_updated_at_idx",
        ),
        migrations.RemoveIndex(
            model_name="cartitem",
            name="cart_cartit_cart_id_cabd5a_btree",
        ),
    ]
