# Generated by Django 5.2.3 on 2025-06-21 14:53

import django.db.models.deletion
import parler.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0015_recreate_translatable_history_models'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='orderhistorytranslation',
            options={'default_permissions': (), 'managed': True, 'verbose_name': 'Order History Translation'},
        ),
        migrations.AlterModelOptions(
            name='orderitemhistorytranslation',
            options={'default_permissions': (), 'managed': True, 'verbose_name': 'Order Item History Translation'},
        ),
        migrations.AlterField(
            model_name='orderhistorytranslation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='orderhistorytranslation',
            name='master',
            field=parler.fields.TranslationsForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='order.orderhistory'),
        ),
        migrations.AlterField(
            model_name='orderitemhistorytranslation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='orderitemhistorytranslation',
            name='master',
            field=parler.fields.TranslationsForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='order.orderitemhistory'),
        ),
        migrations.AlterModelTable(
            name='orderhistorytranslation',
            table='order_orderhistory_translation',
        ),
        migrations.AlterModelTable(
            name='orderitemhistorytranslation',
            table='order_orderitemhistory_translation',
        ),
    ]
