# Generated by Django 5.2 on 2025-04-17 13:12

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cart', '0007_remove_cart_cart_created_at_idx_and_more'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='cart',
            name='session_key',
            field=models.CharField(blank=True, help_text='Session key for guest users', max_length=40, null=True, verbose_name='Session Key'),
        ),
        migrations.AddConstraint(
            model_name='cart',
            constraint=models.UniqueConstraint(condition=models.Q(('session_key__isnull', False)), fields=('session_key',), name='unique_session_cart'),
        ),
    ]
