# Generated by Django 5.2.3 on 2025-06-21 14:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('blog', '0026_remove_blogpost_blog_post_status_ix_and_more'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='blogtag',
            managers=[
            ],
        ),
    ]
