# Generated by Django 5.2.1 on 2025-05-14 20:33

import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('country', '0005_alter_country_alpha_2_alter_country_alpha_3_and_more'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='country',
            name='country_cou_alpha_2_398383_idx',
        ),
        migrations.RemoveIndex(
            model_name='country',
            name='country_cou_alpha_3_c813cd_idx',
        ),
        migrations.RenameIndex(
            model_name='country',
            new_name='country_created_at_ix',
            old_name='country_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='country',
            new_name='country_updated_at_ix',
            old_name='country_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='country',
            new_name='country_sort_order_ix',
            old_name='country_sort_order_idx',
        ),
        migrations.AddIndex(
            model_name='country',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['alpha_2'], name='country_alpha_2_ix'),
        ),
        migrations.AddIndex(
            model_name='country',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['alpha_3'], name='country_alpha_3_ix'),
        ),
        migrations.AddIndex(
            model_name='country',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['iso_cc'], name='country_iso_cc_ix'),
        ),
        migrations.AddIndex(
            model_name='country',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['phone_code'], name='country_phone_code_ix'),
        ),
    ]
