# Generated by Django 5.0.4 on 2024-04-11 17:27
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("country", "0003_alter_country_options"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="country",
            options={
                "ordering": ["sort_order"],
                "verbose_name": "Country",
                "verbose_name_plural": "Countries",
            },
        ),
    ]
