# Generated by Django 5.2.6 on 2025-09-29 22:54

import django.core.serializers.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0016_alter_orderhistorytranslation_options_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='metadata',
            field=models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder),
        ),
        migrations.AddField(
            model_name='order',
            name='private_metadata',
            field=models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder),
        ),
    ]
