# Generated by Django 5.2.2 on 2025-06-07 17:12

import django.contrib.postgres.indexes
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notification', '0009_remove_notificationuser_notificatio_user_id_f8507b_idx_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='category',
            field=models.CharField(choices=[('order', 'Order'), ('payment', 'Payment'), ('shipping', 'Shipping'), ('cart', 'Cart'), ('product', 'Product'), ('account', 'Account'), ('security', 'Security'), ('promotion', 'Promotion'), ('system', 'System'), ('review', 'Review'), ('wishlist', 'Wishlist'), ('support', 'Support'), ('newsletter', 'Newsletter'), ('recommendation', 'Recommendation')], db_index=True, default='system', max_length=50, verbose_name='Category'),
        ),
        migrations.AddField(
            model_name='notification',
            name='notification_type',
            field=models.CharField(blank=True, help_text='Specific type of notification for categorization', max_length=100, verbose_name='Notification Type'),
        ),
        migrations.AddField(
            model_name='notification',
            name='priority',
            field=models.CharField(choices=[('low', 'Low Priority'), ('normal', 'Normal Priority'), ('high', 'High Priority'), ('urgent', 'Urgent Priority'), ('critical', 'Critical Priority')], db_index=True, default='normal', max_length=20, verbose_name='Priority'),
        ),
        migrations.AddIndex(
            model_name='notification',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['category'], name='notification_category_ix'),
        ),
        migrations.AddIndex(
            model_name='notification',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['priority'], name='notification_priority_ix'),
        ),
        migrations.AddIndex(
            model_name='notification',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['notification_type'], name='notification_type_ix'),
        ),
        migrations.AddIndex(
            model_name='notification',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['category', 'priority'], name='notification_cat_pri_ix'),
        ),
    ]
