# Generated by Django 5.0.4 on 2024-04-17 15:40
from decimal import Decimal

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("vat", "0003_vat_vat_vat_value_d54579_btree"),
    ]

    operations = [
        migrations.AlterField(
            model_name="vat",
            name="value",
            field=models.DecimalField(
                decimal_places=1,
                default=Decimal("0"),
                max_digits=11,
                verbose_name="Value",
            ),
        ),
    ]
