# Generated by Django 5.2.1 on 2025-05-23 14:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('product', '0022_alter_productfavourite_options_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='productimage',
            name='thumbnail',
        ),
    ]
